/****************************************************************************
** Meta object code from reading C++ file 'thumbnailload.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/thumbnailload.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'thumbnailload.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LoadImage_t {
    QByteArrayData data[22];
    char stringdata0[277];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadImage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadImage_t qt_meta_stringdata_LoadImage = {
    {
QT_MOC_LITERAL(0, 0, 9), // "LoadImage"
QT_MOC_LITERAL(1, 10, 16), // "callQmlRefeshImg"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 14), // "loadThumbnails"
QT_MOC_LITERAL(4, 43, 4), // "list"
QT_MOC_LITERAL(5, 48, 13), // "loadThumbnail"
QT_MOC_LITERAL(6, 62, 4), // "path"
QT_MOC_LITERAL(7, 67, 12), // "catThumbnail"
QT_MOC_LITERAL(8, 80, 18), // "onImageFileChanged"
QT_MOC_LITERAL(9, 99, 12), // "isMultiImage"
QT_MOC_LITERAL(10, 112, 7), // "isExist"
QT_MOC_LITERAL(11, 120, 17), // "getFitWindowScale"
QT_MOC_LITERAL(12, 138, 11), // "WindowWidth"
QT_MOC_LITERAL(13, 150, 12), // "WindowHeight"
QT_MOC_LITERAL(14, 163, 11), // "imageIsNull"
QT_MOC_LITERAL(15, 175, 13), // "getImageWidth"
QT_MOC_LITERAL(16, 189, 14), // "getImageHeight"
QT_MOC_LITERAL(17, 204, 23), // "getrealWidthHeightRatio"
QT_MOC_LITERAL(18, 228, 18), // "setMultiFrameIndex"
QT_MOC_LITERAL(19, 247, 5), // "index"
QT_MOC_LITERAL(20, 253, 21), // "setReverseHeightWidth"
QT_MOC_LITERAL(21, 275, 1) // "b"

    },
    "LoadImage\0callQmlRefeshImg\0\0loadThumbnails\0"
    "list\0loadThumbnail\0path\0catThumbnail\0"
    "onImageFileChanged\0isMultiImage\0isExist\0"
    "getFitWindowScale\0WindowWidth\0"
    "WindowHeight\0imageIsNull\0getImageWidth\0"
    "getImageHeight\0getrealWidthHeightRatio\0"
    "setMultiFrameIndex\0index\0setReverseHeightWidth\0"
    "b"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadImage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   90,    2, 0x0a /* Public */,
       5,    1,   93,    2, 0x0a /* Public */,
       7,    1,   96,    2, 0x0a /* Public */,
       8,    3,   99,    2, 0x0a /* Public */,
       8,    2,  106,    2, 0x2a /* Public | MethodCloned */,
       8,    1,  111,    2, 0x2a /* Public | MethodCloned */,

 // methods: name, argc, parameters, tag, flags
      11,    3,  114,    2, 0x02 /* Public */,
      14,    1,  121,    2, 0x02 /* Public */,
      15,    1,  124,    2, 0x02 /* Public */,
      16,    1,  127,    2, 0x02 /* Public */,
      17,    1,  130,    2, 0x02 /* Public */,
      18,    1,  133,    2, 0x02 /* Public */,
      18,    0,  136,    2, 0x22 /* Public | MethodCloned */,
      20,    1,  137,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QStringList,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QStringList,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,    6,    9,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    6,    9,
    QMetaType::Void, QMetaType::QString,    6,

 // methods: parameters
    QMetaType::Double, QMetaType::QString, QMetaType::Double, QMetaType::Double,    6,   12,   13,
    QMetaType::Bool, QMetaType::QString,    6,
    QMetaType::Int, QMetaType::QString,    6,
    QMetaType::Int, QMetaType::QString,    6,
    QMetaType::Double, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   21,

       0        // eod
};

void LoadImage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LoadImage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->callQmlRefeshImg(); break;
        case 1: _t->loadThumbnails((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 2: _t->loadThumbnail((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->catThumbnail((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 4: _t->onImageFileChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 5: _t->onImageFileChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->onImageFileChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: { double _r = _t->getFitWindowScale((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->imageIsNull((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->getImageWidth((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->getImageHeight((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 11: { double _r = _t->getrealWidthHeightRatio((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setMultiFrameIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->setMultiFrameIndex(); break;
        case 14: _t->setReverseHeightWidth((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LoadImage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LoadImage::callQmlRefeshImg)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LoadImage::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_LoadImage.data,
    qt_meta_data_LoadImage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadImage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadImage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadImage.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LoadImage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void LoadImage::callQmlRefeshImg()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
