/****************************************************************************
** Meta object code from reading C++ file 'filecontrol.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/filecontrol.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filecontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileControl_t {
    QByteArrayData data[86];
    char stringdata0[1116];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileControl_t qt_meta_stringdata_FileControl = {
    {
QT_MOC_LITERAL(0, 0, 11), // "FileControl"
QT_MOC_LITERAL(1, 12, 23), // "requestImageFileChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 8), // "filePath"
QT_MOC_LITERAL(4, 46, 12), // "isMultiImage"
QT_MOC_LITERAL(5, 59, 7), // "isExist"
QT_MOC_LITERAL(6, 67, 16), // "imageFileChanged"
QT_MOC_LITERAL(7, 84, 10), // "getDirPath"
QT_MOC_LITERAL(8, 95, 4), // "path"
QT_MOC_LITERAL(9, 100, 15), // "getDirImagePath"
QT_MOC_LITERAL(10, 116, 10), // "removeList"
QT_MOC_LITERAL(11, 127, 8), // "pathlist"
QT_MOC_LITERAL(12, 136, 5), // "index"
QT_MOC_LITERAL(13, 142, 7), // "isImage"
QT_MOC_LITERAL(14, 150, 12), // "setWallpaper"
QT_MOC_LITERAL(15, 163, 7), // "imgPath"
QT_MOC_LITERAL(16, 171, 15), // "deleteImagePath"
QT_MOC_LITERAL(17, 187, 20), // "displayinFileManager"
QT_MOC_LITERAL(18, 208, 9), // "copyImage"
QT_MOC_LITERAL(19, 218, 8), // "copyText"
QT_MOC_LITERAL(20, 227, 3), // "str"
QT_MOC_LITERAL(21, 231, 11), // "isRotatable"
QT_MOC_LITERAL(22, 243, 10), // "isCanWrite"
QT_MOC_LITERAL(23, 254, 11), // "isCanDelete"
QT_MOC_LITERAL(24, 266, 6), // "isFile"
QT_MOC_LITERAL(25, 273, 23), // "parseCommandlineGetPath"
QT_MOC_LITERAL(26, 297, 14), // "isDynamicImage"
QT_MOC_LITERAL(27, 312, 19), // "isNormalStaticImage"
QT_MOC_LITERAL(28, 332, 10), // "rotateFile"
QT_MOC_LITERAL(29, 343, 11), // "rotateAngel"
QT_MOC_LITERAL(30, 355, 20), // "slotRotatePixCurrent"
QT_MOC_LITERAL(31, 376, 20), // "isReverseHeightWidth"
QT_MOC_LITERAL(32, 397, 12), // "currentAngle"
QT_MOC_LITERAL(33, 410, 15), // "slotGetFileName"
QT_MOC_LITERAL(34, 426, 21), // "slotGetFileNameSuffix"
QT_MOC_LITERAL(35, 448, 11), // "slotGetInfo"
QT_MOC_LITERAL(36, 460, 3), // "key"
QT_MOC_LITERAL(37, 464, 14), // "slotFileReName"
QT_MOC_LITERAL(38, 479, 4), // "name"
QT_MOC_LITERAL(39, 484, 8), // "filepath"
QT_MOC_LITERAL(40, 493, 8), // "isSuffix"
QT_MOC_LITERAL(41, 502, 9), // "renameOne"
QT_MOC_LITERAL(42, 512, 7), // "oldPath"
QT_MOC_LITERAL(43, 520, 7), // "newPath"
QT_MOC_LITERAL(44, 528, 11), // "getNamePath"
QT_MOC_LITERAL(45, 540, 7), // "newName"
QT_MOC_LITERAL(46, 548, 14), // "slotFileSuffix"
QT_MOC_LITERAL(47, 563, 3), // "ret"
QT_MOC_LITERAL(48, 567, 15), // "setCurrentImage"
QT_MOC_LITERAL(49, 583, 20), // "setCurrentFrameIndex"
QT_MOC_LITERAL(50, 604, 20), // "getCurrentImageWidth"
QT_MOC_LITERAL(51, 625, 21), // "getCurrentImageHeight"
QT_MOC_LITERAL(52, 647, 17), // "getFitWindowScale"
QT_MOC_LITERAL(53, 665, 11), // "WindowWidth"
QT_MOC_LITERAL(54, 677, 12), // "WindowHeight"
QT_MOC_LITERAL(55, 690, 13), // "isShowToolTip"
QT_MOC_LITERAL(56, 704, 15), // "showPrintDialog"
QT_MOC_LITERAL(57, 720, 14), // "getConfigValue"
QT_MOC_LITERAL(58, 735, 5), // "group"
QT_MOC_LITERAL(59, 741, 12), // "defaultValue"
QT_MOC_LITERAL(60, 754, 14), // "setConfigValue"
QT_MOC_LITERAL(61, 769, 5), // "value"
QT_MOC_LITERAL(62, 775, 12), // "getlastWidth"
QT_MOC_LITERAL(63, 788, 13), // "getlastHeight"
QT_MOC_LITERAL(64, 802, 15), // "setSettingWidth"
QT_MOC_LITERAL(65, 818, 5), // "width"
QT_MOC_LITERAL(66, 824, 16), // "setSettingHeight"
QT_MOC_LITERAL(67, 841, 6), // "height"
QT_MOC_LITERAL(68, 848, 19), // "setEnableNavigation"
QT_MOC_LITERAL(69, 868, 1), // "b"
QT_MOC_LITERAL(70, 870, 18), // "isEnableNavigation"
QT_MOC_LITERAL(71, 889, 11), // "saveSetting"
QT_MOC_LITERAL(72, 901, 21), // "isSupportSetWallpaper"
QT_MOC_LITERAL(73, 923, 11), // "isCheckOnly"
QT_MOC_LITERAL(74, 935, 11), // "isCanRename"
QT_MOC_LITERAL(75, 947, 13), // "isCanReadable"
QT_MOC_LITERAL(76, 961, 10), // "isSvgImage"
QT_MOC_LITERAL(77, 972, 12), // "imageIsExist"
QT_MOC_LITERAL(78, 985, 13), // "getImageCount"
QT_MOC_LITERAL(79, 999, 15), // "resetImageFiles"
QT_MOC_LITERAL(80, 1015, 9), // "filePaths"
QT_MOC_LITERAL(81, 1025, 14), // "getCompanyLogo"
QT_MOC_LITERAL(82, 1040, 17), // "showShortcutPanel"
QT_MOC_LITERAL(83, 1058, 13), // "windowCenterX"
QT_MOC_LITERAL(84, 1072, 13), // "windowCenterY"
QT_MOC_LITERAL(85, 1086, 29) // "terminateShortcutPanelProcess"

    },
    "FileControl\0requestImageFileChanged\0"
    "\0filePath\0isMultiImage\0isExist\0"
    "imageFileChanged\0getDirPath\0path\0"
    "getDirImagePath\0removeList\0pathlist\0"
    "index\0isImage\0setWallpaper\0imgPath\0"
    "deleteImagePath\0displayinFileManager\0"
    "copyImage\0copyText\0str\0isRotatable\0"
    "isCanWrite\0isCanDelete\0isFile\0"
    "parseCommandlineGetPath\0isDynamicImage\0"
    "isNormalStaticImage\0rotateFile\0"
    "rotateAngel\0slotRotatePixCurrent\0"
    "isReverseHeightWidth\0currentAngle\0"
    "slotGetFileName\0slotGetFileNameSuffix\0"
    "slotGetInfo\0key\0slotFileReName\0name\0"
    "filepath\0isSuffix\0renameOne\0oldPath\0"
    "newPath\0getNamePath\0newName\0slotFileSuffix\0"
    "ret\0setCurrentImage\0setCurrentFrameIndex\0"
    "getCurrentImageWidth\0getCurrentImageHeight\0"
    "getFitWindowScale\0WindowWidth\0"
    "WindowHeight\0isShowToolTip\0showPrintDialog\0"
    "getConfigValue\0group\0defaultValue\0"
    "setConfigValue\0value\0getlastWidth\0"
    "getlastHeight\0setSettingWidth\0width\0"
    "setSettingHeight\0height\0setEnableNavigation\0"
    "b\0isEnableNavigation\0saveSetting\0"
    "isSupportSetWallpaper\0isCheckOnly\0"
    "isCanRename\0isCanReadable\0isSvgImage\0"
    "imageIsExist\0getImageCount\0resetImageFiles\0"
    "filePaths\0getCompanyLogo\0showShortcutPanel\0"
    "windowCenterX\0windowCenterY\0"
    "terminateShortcutPanelProcess"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      66,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,  344,    2, 0x06 /* Public */,
       1,    2,  351,    2, 0x26 /* Public | MethodCloned */,
       1,    1,  356,    2, 0x26 /* Public | MethodCloned */,
       6,    3,  359,    2, 0x06 /* Public */,
       6,    2,  366,    2, 0x26 /* Public | MethodCloned */,
       6,    1,  371,    2, 0x26 /* Public | MethodCloned */,

 // methods: name, argc, parameters, tag, flags
       7,    1,  374,    2, 0x02 /* Public */,
       9,    1,  377,    2, 0x02 /* Public */,
      10,    2,  380,    2, 0x02 /* Public */,
      13,    1,  385,    2, 0x02 /* Public */,
      14,    1,  388,    2, 0x02 /* Public */,
      16,    1,  391,    2, 0x02 /* Public */,
      17,    1,  394,    2, 0x02 /* Public */,
      18,    1,  397,    2, 0x02 /* Public */,
      19,    1,  400,    2, 0x02 /* Public */,
      21,    1,  403,    2, 0x02 /* Public */,
      22,    1,  406,    2, 0x02 /* Public */,
      23,    1,  409,    2, 0x02 /* Public */,
      24,    1,  412,    2, 0x02 /* Public */,
      25,    1,  415,    2, 0x02 /* Public */,
      26,    1,  418,    2, 0x02 /* Public */,
      27,    1,  421,    2, 0x02 /* Public */,
      28,    2,  424,    2, 0x02 /* Public */,
      30,    0,  429,    2, 0x02 /* Public */,
      31,    0,  430,    2, 0x02 /* Public */,
      32,    0,  431,    2, 0x02 /* Public */,
      33,    1,  432,    2, 0x02 /* Public */,
      34,    1,  435,    2, 0x02 /* Public */,
      35,    2,  438,    2, 0x02 /* Public */,
      37,    3,  443,    2, 0x02 /* Public */,
      37,    2,  450,    2, 0x22 /* Public | MethodCloned */,
      41,    3,  455,    2, 0x02 /* Public */,
      44,    2,  462,    2, 0x02 /* Public */,
      46,    2,  467,    2, 0x02 /* Public */,
      46,    1,  472,    2, 0x22 /* Public | MethodCloned */,
      48,    1,  475,    2, 0x02 /* Public */,
      49,    1,  478,    2, 0x02 /* Public */,
      50,    0,  481,    2, 0x02 /* Public */,
      51,    0,  482,    2, 0x02 /* Public */,
      52,    2,  483,    2, 0x02 /* Public */,
      52,    1,  488,    2, 0x22 /* Public | MethodCloned */,
      52,    0,  491,    2, 0x22 /* Public | MethodCloned */,
      55,    2,  492,    2, 0x02 /* Public */,
      56,    1,  497,    2, 0x02 /* Public */,
      57,    3,  500,    2, 0x02 /* Public */,
      57,    2,  507,    2, 0x22 /* Public | MethodCloned */,
      60,    3,  512,    2, 0x02 /* Public */,
      62,    0,  519,    2, 0x02 /* Public */,
      63,    0,  520,    2, 0x02 /* Public */,
      64,    1,  521,    2, 0x02 /* Public */,
      66,    1,  524,    2, 0x02 /* Public */,
      68,    1,  527,    2, 0x02 /* Public */,
      70,    0,  530,    2, 0x02 /* Public */,
      71,    0,  531,    2, 0x02 /* Public */,
      72,    1,  532,    2, 0x02 /* Public */,
      73,    0,  535,    2, 0x02 /* Public */,
      74,    1,  536,    2, 0x02 /* Public */,
      75,    1,  539,    2, 0x02 /* Public */,
      76,    1,  542,    2, 0x02 /* Public */,
       4,    1,  545,    2, 0x02 /* Public */,
      77,    1,  548,    2, 0x02 /* Public */,
      78,    1,  551,    2, 0x02 /* Public */,
      79,    1,  554,    2, 0x02 /* Public */,
      81,    0,  557,    2, 0x02 /* Public */,
      82,    2,  558,    2, 0x02 /* Public */,
      85,    0,  563,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,

 // methods: parameters
    QMetaType::QString, QMetaType::QString,    8,
    QMetaType::QStringList, QMetaType::QString,    8,
    QMetaType::QStringList, QMetaType::QStringList, QMetaType::Int,   11,   12,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::QString, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int,    8,   29,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Int,
    QMetaType::QString, QMetaType::QString,    8,
    QMetaType::QString, QMetaType::QString,    8,
    QMetaType::QString, QMetaType::QString, QMetaType::QString,   36,    8,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::Bool,   38,   39,   40,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   38,   39,
    QMetaType::QStringList, QMetaType::QStringList, QMetaType::QString, QMetaType::QString,   11,   42,   43,
    QMetaType::QString, QMetaType::QString, QMetaType::QString,   42,   45,
    QMetaType::QString, QMetaType::QString, QMetaType::Bool,    8,   47,
    QMetaType::QString, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Int,
    QMetaType::Int,
    QMetaType::Double, QMetaType::Double, QMetaType::Double,   53,   54,
    QMetaType::Double, QMetaType::Double,   53,
    QMetaType::Double,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   42,   38,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::QVariant, QMetaType::QString, QMetaType::QString, QMetaType::QVariant,   58,   36,   59,
    QMetaType::QVariant, QMetaType::QString, QMetaType::QString,   58,   36,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QVariant,   58,   36,   61,
    QMetaType::Int,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   65,
    QMetaType::Void, QMetaType::Int,   67,
    QMetaType::Void, QMetaType::Bool,   69,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Bool, QMetaType::QString,    8,
    QMetaType::Int, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QStringList,   80,
    QMetaType::QUrl,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   83,   84,
    QMetaType::Void,

       0        // eod
};

void FileControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestImageFileChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 1: _t->requestImageFileChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->requestImageFileChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->imageFileChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 4: _t->imageFileChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->imageFileChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: { QString _r = _t->getDirPath((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QStringList _r = _t->getDirImagePath((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 8: { QStringList _r = _t->removeList((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->isImage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setWallpaper((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: { bool _r = _t->deleteImagePath((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->displayinFileManager((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->copyImage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->copyText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: { bool _r = _t->isRotatable((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->isCanWrite((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isCanDelete((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->parseCommandlineGetPath((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->isDynamicImage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->isNormalStaticImage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->rotateFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->slotRotatePixCurrent(); break;
        case 24: { bool _r = _t->isReverseHeightWidth();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { int _r = _t->currentAngle();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 26: { QString _r = _t->slotGetFileName((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->slotGetFileNameSuffix((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 28: { QString _r = _t->slotGetInfo((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->slotFileReName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->slotFileReName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { QStringList _r = _t->renameOne((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 32: { QString _r = _t->getNamePath((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 33: { QString _r = _t->slotFileSuffix((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 34: { QString _r = _t->slotFileSuffix((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setCurrentImage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 36: _t->setCurrentFrameIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: { int _r = _t->getCurrentImageWidth();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 38: { int _r = _t->getCurrentImageHeight();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 39: { double _r = _t->getFitWindowScale((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 40: { double _r = _t->getFitWindowScale((*reinterpret_cast< double(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 41: { double _r = _t->getFitWindowScale();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 42: { bool _r = _t->isShowToolTip((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->showPrintDialog((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 44: { QVariant _r = _t->getConfigValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 45: { QVariant _r = _t->getConfigValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 46: _t->setConfigValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 47: { int _r = _t->getlastWidth();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 48: { int _r = _t->getlastHeight();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 49: _t->setSettingWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 50: _t->setSettingHeight((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 51: _t->setEnableNavigation((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 52: { bool _r = _t->isEnableNavigation();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->saveSetting(); break;
        case 54: { bool _r = _t->isSupportSetWallpaper((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 55: { bool _r = _t->isCheckOnly();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 56: { bool _r = _t->isCanRename((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 57: { bool _r = _t->isCanReadable((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 58: { bool _r = _t->isSvgImage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 59: { bool _r = _t->isMultiImage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 60: { bool _r = _t->imageIsExist((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 61: { int _r = _t->getImageCount((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 62: _t->resetImageFiles((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 63: { QUrl _r = _t->getCompanyLogo();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 64: _t->showShortcutPanel((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 65: _t->terminateShortcutPanelProcess(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileControl::*)(const QString & , bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileControl::requestImageFileChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileControl::*)(const QString & , bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileControl::imageFileChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FileControl.data,
    qt_meta_data_FileControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileControl.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FileControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 66)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 66;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 66)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 66;
    }
    return _id;
}

// SIGNAL 0
void FileControl::requestImageFileChanged(const QString & _t1, bool _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 3
void FileControl::imageFileChanged(const QString & _t1, bool _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
